from pydoc import describe
import numpy as np
# Copyright © 2005-2019, NumPy Developers.

import pandas as pd
# Copyright (c) 2008-2012, AQR Capital Management, LLC, Lambda Foundry, Inc. and PyData Development Team

from sklearn.impute import KNNImputer
import time
import os
from sklearn.preprocessing import StandardScaler


data = pd.read_csv('#filnavn#.csv', low_memory=False)


data = data.rename(columns={'company_details.employees':'ansatte'})
data['sum_inntekter'] = sum_inntekter
data['lonnskostnad'] = lonnskostnad
data['fravalg_revisjon'] = data['fravalg_revisjon'].astype('category')
data['utarbeidet_regnskapsforer'] = data['utarbeidet_regnskapsforer'].astype('category')
data['bistand_regnskapsforer'] = data['bistand_regnskapsforer'].astype('category')
data['dummy_to_dl'] = data['dummy_to_dl'].astype('category')



def KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data,do_standardize_vector_space):
    # All variables that are imputed are treated as continious variables
    
    # var_mis_con: CONTINIOUS variables with missing values to be imputed
    # var_mis_cat: CATEGORICAL variables with missing values to be imputed
    # var_use: variables used in the vector space for the kNN algorithm for imputation 
    
    # n_neighbors_con: number of neighboring samples to use for imputation for continious variables
    # For categorical variables, n_neighbors is one for selecting the value of the one closest

    # Making sure that index is reset.
    # If not, the process will fail
    data = data.reset_index(drop=True) # Reset index

    # Checking that variables used for imputation do not have missing values
    temp = np.sum(pd.isnull(data[var_use]))
    temp = temp[temp!=0].index.tolist()
    if len(temp)>=1:
        for i in temp:
            print('-------------------------')
            print('ERROR: Variable \'{}\' is used for k-NN imputation. However, it has missing values. Missing values are set to zero in the k-NN imputation, so consider not including this variable.'.format(i))
        print('-------------------------')
        print('Imputing failed. No values imputed.')
        print('-------------------------')

    elif len(temp)==0:
        # Imputing variable, one at a time

        # Creating vector space used for the kNN-algorithm
        data_var_use = data[var_use]

        # Standardizing the vector space to a mean of 0 and 
        # standard deviation of 1 for each variable, respectively
        if do_standardize_vector_space:
            data_var_use = pd.DataFrame(StandardScaler().fit_transform(data_var_use),columns=data_var_use.columns)

        # First, continious variables
        for i in var_mis_con:
            t = time.time()
            print('-------------------------')

            # Making data such that first column is the one to be imputed
            data_for_imputation = pd.concat([data[i],data_var_use],axis=1)
            num_missing_before = np.sum(pd.isnull(data[i]))

            # Imputing
            imputer = KNNImputer(n_neighbors=n_neighbors_con,weights='distance')
            temp = pd.DataFrame(imputer.fit_transform(data_for_imputation))

            # First column is the one imputed, so replacing this one in data
            data[i] = temp[0]

            num_missing_after = np.sum(pd.isnull(data[i]))
            print('Imputed {} instances of missing values for continious variable \'{}\''.format(num_missing_before-num_missing_after,i))
            print('Elapset time: {} minutes'.format(np.round((time.time()-t)/60,2)))
            if num_missing_after!=0:
                print('ERROR: Imputation failed: still {} instances of missing values for variable {}'.format(num_missing_after,i))

        # Second, categorical variables including dummies
        for i in var_mis_cat:
            t = time.time()
            print('-------------------------')

            # Making data such that first column is the one to be imputed
            data_for_imputation = pd.concat([data[i],data_var_use],axis=1)
            num_missing_before = np.sum(pd.isnull(data[i]))
            num_cat_values_before = len(data[i][pd.isnull(data[i])==False].unique())
            
            # Imputing
            imputer = KNNImputer(n_neighbors=1)
            temp = pd.DataFrame(imputer.fit_transform(data_for_imputation))

            # First column is the one imputed, so replacing this one in data
            data[i] = temp[0]

            num_missing_after = np.sum(pd.isnull(data[i]))
            num_cat_values_after = len(data[i][pd.isnull(data[i])==False].unique())

            print('Imputed {} instances of missing values for categorical variable \'{}\''.format(num_missing_before-num_missing_after,i))
            print('Elapset time: {} minutes'.format(np.round((time.time()-t)/60,2)))
            if num_cat_values_before!=num_cat_values_after:
                print('ERROR: number of categorical values before and after impotation are {} and {}, respectively.'.format(num_cat_values_before,num_cat_values_after))
            else:
                print('Number of categorical values for variable \'{}\': {}'.format(i,num_cat_values_after))
            if num_missing_after!=0:
                print('ERROR: Imputation failed: still {} instances of missing values for variable {}'.format(num_missing_after,i))

        print('-------------------------')

    return data

###########################
## Utfører kNN år for år ##
###########################



## 2006 ##

data06 = data[data['regnaar']==2006]
data06 = data06.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True


# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data06[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data06[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data06[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data06[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data06[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data06[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data06[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data06[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data06[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data06[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data06[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data06[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data06[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data06[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data06[var_mis_cat[6]]))))
print('-------------------------')



# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data06 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data06,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data06[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data06[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data06[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data06[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data06[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data06[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data06[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data06[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data06[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data06[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data06[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data06[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data06[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data06[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data06[var_mis_cat[6]]))))
print('-------------------------')


#######2007#######

data07 = data[data['regnaar']==2007]
data07 = data07.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True


# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',

]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data07[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data07[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data07[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data07[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data07[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data07[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data07[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data07[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data07[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data07[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data07[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data07[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data07[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data07[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data07[var_mis_cat[6]]))))
print('-------------------------')

#########

# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data07 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data07,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data07[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data07[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data07[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data07[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data07[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data07[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data07[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data07[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data07[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data07[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data07[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data07[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data07[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data07[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data07[var_mis_cat[6]]))))
print('-------------------------')

#######2008#######

data08 = data[data['regnaar']==2008]
data08 = data08.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data08[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data08[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data08[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data08[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data08[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data08[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data08[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data08[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data08[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data08[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data08[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data08[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data08[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data08[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data08[var_mis_cat[6]]))))
print('-------------------------')

#########


# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data08 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data08,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data08[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data08[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data08[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data08[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data08[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data08[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data08[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data08[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data08[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data08[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data08[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data08[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data08[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data08[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data08[var_mis_cat[6]]))))
print('-------------------------')

#######2009#######
data09 = data[data['regnaar']==2009]
data09 = data09.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data09[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data09[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data09[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data09[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data09[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data09[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data09[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data09[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data09[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data09[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data09[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data09[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data09[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data09[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data09[var_mis_cat[6]]))))
print('-------------------------')


# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data09 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data09,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data09[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data09[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data09[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data09[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data09[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data09[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data09[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data09[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data09[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data09[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data09[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data09[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data09[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data09[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data09[var_mis_cat[6]]))))
print('-------------------------')


#######2010#######

data10 = data[data['regnaar']==2010]
data10 = data10.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data10[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data10[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data10[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data10[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data10[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data10[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data10[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data10[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data10[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data10[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data10[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data10[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data10[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data10[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data10[var_mis_cat[6]]))))
print('-------------------------')



# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data10 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data10,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data10[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data10[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data10[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data10[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data10[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data10[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data10[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data10[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data10[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data10[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data10[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data10[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data10[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data10[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data10[var_mis_cat[6]]))))
print('-------------------------')

#######2011#######
data11 = data[data['regnaar']==2011]
data11 = data11.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data11[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data11[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data11[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data11[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data11[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data11[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data11[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data11[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data11[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data11[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data11[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data11[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data11[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data11[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data11[var_mis_cat[6]]))))
print('-------------------------')



# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data11 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data11,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data11[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data11[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data11[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data11[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data11[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data11[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data11[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data11[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data11[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data11[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data11[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data11[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data11[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data11[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data11[var_mis_cat[6]]))))
print('-------------------------')

#######2012#######
data12 = data[data['regnaar']==2012]
data12 = data12.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data12[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data12[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data12[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data12[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data12[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data12[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data12[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data12[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data12[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data12[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data12[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data12[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data12[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data12[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data12[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data12[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data12[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data12[var_mis_cat[9]]))))
print('-------------------------')

#########

# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data12 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data12,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data12[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data12[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data12[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data12[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data12[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data12[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data12[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data12[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data12[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data12[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data12[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data12[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data12[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data12[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data12[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data12[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data12[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data12[var_mis_cat[9]]))))
print('-------------------------')

#######2013#######
data13 = data[data['regnaar']==2013]
data13 = data13.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data13[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data13[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data13[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data13[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data13[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data13[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data13[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data13[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data13[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data13[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data13[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data13[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data13[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data13[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data13[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data13[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data13[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data13[var_mis_cat[9]]))))
print('-------------------------')


#########

# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data13 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data13,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data13[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data13[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data13[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data13[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data13[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data13[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data13[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data13[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data13[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data13[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data13[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data13[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data13[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data13[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data13[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data13[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data13[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data13[var_mis_cat[9]]))))
print('-------------------------')

#######2014#######
data14 = data[data['regnaar']==2014]
data14 = data14.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'dl_share_aksje',
    'sl_share_aksje',
    'styre_share_aksje',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
    'aksjonaerer_antall_end',
    'aksjonaerer_antall_beginning',
    'aksjonaerer_mean_end',
    'aksjonaerer_std_end',
    'aksjonaerer_max_end',
    'aksjonaerer_min_end',
    'aksjonaerer_mean_beginning',
    'aksjonaerer_std_beginning',
    'aksjonaerer_max_beginning',
    'aksjonaerer_min_beginning',
    'antall_styremedlemmer_ikke_aksjonaer',
    'andel_styremedlemmer_ikke_aksjonaer',
    'aksjonaerer_andel_fylkesnr',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data14[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data14[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data14[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data14[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data14[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data14[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data14[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data14[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data14[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data14[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data14[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data14[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data14[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data14[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data14[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data14[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data14[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data14[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data14[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data14[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data14[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data14[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data14[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data14[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data14[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data14[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data14[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data14[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data14[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data14[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data14[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data14[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data14[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data14[var_mis_cat[9]]))))
print('-------------------------')


# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data14 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data14,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data14[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data14[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data14[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data14[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data14[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data14[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data14[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data14[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data14[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data14[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data14[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data14[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data14[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data14[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data14[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data14[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data14[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data14[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data14[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data14[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data14[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data14[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data14[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data14[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data14[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data14[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data14[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data14[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data14[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data14[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data14[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data14[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data14[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data14[var_mis_cat[9]]))))
print('-------------------------')


#######2015#######
data15 = data[data['regnaar']==2015]
data15 = data15.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'dl_share_aksje',
    'sl_share_aksje',
    'styre_share_aksje',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
    'aksjonaerer_antall_end',
    'aksjonaerer_antall_beginning',
    'aksjonaerer_mean_end',
    'aksjonaerer_std_end',
    'aksjonaerer_max_end',
    'aksjonaerer_min_end',
    'aksjonaerer_mean_beginning',
    'aksjonaerer_std_beginning',
    'aksjonaerer_max_beginning',
    'aksjonaerer_min_beginning',
    'antall_styremedlemmer_ikke_aksjonaer',
    'andel_styremedlemmer_ikke_aksjonaer',
    'aksjonaerer_andel_fylkesnr',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data15[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data15[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data15[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data15[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data15[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data15[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data15[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data15[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data15[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data15[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data15[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data15[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data15[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data15[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data15[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data15[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data15[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data15[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data15[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data15[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data15[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data15[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data15[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data15[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data15[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data15[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data15[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data15[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data15[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data15[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data15[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data15[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data15[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data15[var_mis_cat[9]]))))
print('-------------------------')


# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data15 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data15,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data15[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data15[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data15[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data15[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data15[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data15[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data15[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data15[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data15[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data15[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data15[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data15[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data15[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data15[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data15[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data15[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data15[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data15[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data15[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data15[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data15[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data15[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data15[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data15[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data15[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data15[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data15[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data15[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data15[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data15[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data15[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data15[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data15[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data15[var_mis_cat[9]]))))
print('-------------------------')

#######2016#######
data16 = data[data['regnaar']==2016]
data16 = data16.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'dl_share_aksje',
    'sl_share_aksje',
    'styre_share_aksje',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
    'aksjonaerer_antall_end',
    'aksjonaerer_antall_beginning',
    'aksjonaerer_mean_end',
    'aksjonaerer_std_end',
    'aksjonaerer_max_end',
    'aksjonaerer_min_end',
    'aksjonaerer_mean_beginning',
    'aksjonaerer_std_beginning',
    'aksjonaerer_max_beginning',
    'aksjonaerer_min_beginning',
    'antall_styremedlemmer_ikke_aksjonaer',
    'andel_styremedlemmer_ikke_aksjonaer',
    'aksjonaerer_andel_fylkesnr',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data16[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data16[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data16[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data16[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data16[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data16[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data16[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data16[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data16[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data16[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data16[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data16[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data16[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data16[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data16[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data16[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data16[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data16[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data16[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data16[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data16[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data16[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data16[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data16[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data16[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data16[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data16[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data16[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data16[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data16[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data16[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data16[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data16[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data16[var_mis_cat[9]]))))
print('-------------------------')

# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data16 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data16,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data16[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data16[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data16[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data16[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data16[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data16[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data16[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data16[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data16[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data16[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data16[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data16[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data16[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data16[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data16[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data16[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data16[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data16[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data16[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data16[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data16[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data16[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data16[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data16[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data16[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data16[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data16[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data16[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data16[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data16[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data16[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data16[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data16[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data16[var_mis_cat[9]]))))
print('-------------------------')

#######2017#######
data17 = data[data['regnaar']==2017]
data17 = data17.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'dl_share_aksje',
    'sl_share_aksje',
    'styre_share_aksje',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
    'aksjonaerer_antall_end',
    'aksjonaerer_antall_beginning',
    'aksjonaerer_mean_end',
    'aksjonaerer_std_end',
    'aksjonaerer_max_end',
    'aksjonaerer_min_end',
    'aksjonaerer_mean_beginning',
    'aksjonaerer_std_beginning',
    'aksjonaerer_max_beginning',
    'aksjonaerer_min_beginning',
    'antall_styremedlemmer_ikke_aksjonaer',
    'andel_styremedlemmer_ikke_aksjonaer',
    'aksjonaerer_andel_fylkesnr',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data17[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data17[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data17[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data17[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data17[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data17[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data17[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data17[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data17[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data17[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data17[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data17[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data17[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data17[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data17[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data17[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data17[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data17[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data17[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data17[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data17[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data17[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data17[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data17[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data17[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data17[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data17[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data17[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data17[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data17[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data17[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data17[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data17[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data17[var_mis_cat[9]]))))
print('-------------------------')



# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data17 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data17,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data17[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data17[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data17[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data17[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data17[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data17[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data17[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data17[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data17[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data17[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data17[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data17[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data17[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data17[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data17[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data17[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data17[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data17[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data17[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data17[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data17[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data17[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data17[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data17[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data17[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data17[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data17[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data17[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data17[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data17[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data17[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data17[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data17[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data17[var_mis_cat[9]]))))
print('-------------------------')

#######2018#######
data18 = data[data['regnaar']==2018]
data18 = data18.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'dl_share_aksje',
    'sl_share_aksje',
    'styre_share_aksje',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
    'aksjonaerer_antall_end',
    'aksjonaerer_antall_beginning',
    'aksjonaerer_mean_end',
    'aksjonaerer_std_end',
    'aksjonaerer_max_end',
    'aksjonaerer_min_end',
    'aksjonaerer_mean_beginning',
    'aksjonaerer_std_beginning',
    'aksjonaerer_max_beginning',
    'aksjonaerer_min_beginning',
    'antall_styremedlemmer_ikke_aksjonaer',
    'andel_styremedlemmer_ikke_aksjonaer',
    'aksjonaerer_andel_fylkesnr',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data18[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data18[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data18[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data18[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data18[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data18[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data18[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data18[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data18[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data18[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data18[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data18[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data18[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data18[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data18[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data18[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data18[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data18[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data18[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data18[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data18[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data18[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data18[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data18[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data18[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data18[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data18[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data18[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data18[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data18[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data18[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data18[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data18[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data18[var_mis_cat[9]]))))
print('-------------------------')



# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data18 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data18,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data18[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data18[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data18[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data18[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data18[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data18[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data18[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data18[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data18[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data18[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data18[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data18[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data18[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data18[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data18[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data18[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data18[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data18[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data18[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data18[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data18[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data18[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data18[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data18[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data18[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data18[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data18[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data18[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data18[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data18[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data18[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data18[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data18[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data18[var_mis_cat[9]]))))
print('-------------------------')


#######2019#######
data19 = data[data['regnaar']==2019]
data19 = data19.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'dl_share_aksje',
    'sl_share_aksje',
    'styre_share_aksje',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
    'aksjonaerer_antall_end',
    'aksjonaerer_antall_beginning',
    'aksjonaerer_mean_end',
    'aksjonaerer_std_end',
    'aksjonaerer_max_end',
    'aksjonaerer_min_end',
    'aksjonaerer_mean_beginning',
    'aksjonaerer_std_beginning',
    'aksjonaerer_max_beginning',
    'aksjonaerer_min_beginning',
    'antall_styremedlemmer_ikke_aksjonaer',
    'andel_styremedlemmer_ikke_aksjonaer',
    'aksjonaerer_andel_fylkesnr',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data19[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data19[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data19[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data19[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data19[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data19[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data19[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data19[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data19[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data19[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data19[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data19[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data19[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data19[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data19[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data19[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data19[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data19[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data19[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data19[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data19[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data19[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data19[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data19[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data19[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data19[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data19[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data19[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data19[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data19[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data19[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data19[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data19[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data19[var_mis_cat[9]]))))
print('-------------------------')



# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data19 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data19,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data19[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data19[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data19[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data19[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data19[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data19[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data19[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data19[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data19[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data19[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data19[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data19[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data19[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data19[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data19[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data19[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data19[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data19[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data19[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data19[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data19[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data19[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data19[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data19[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data19[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data19[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data19[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data19[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data19[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data19[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data19[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data19[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data19[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data19[var_mis_cat[9]]))))
print('-------------------------')


#######2020#######
data20 = data[data['regnaar']==2020]
data20 = data20.reset_index(drop=True)
n_neighbors_con = 3
do_standardize_vector_space = True

# CONTINIOUS variables with missing values to be imputed
var_mis_con = [
    'alder_dl',
    'alder_sl',
    'styre_antall',
    'styre_alder_mean',
    'styre_alder_std',
    'dl_share_aksje',
    'sl_share_aksje',
    'styre_share_aksje',
    'styre_andel_fylkesnr',
    'styre_andel_kjonn_K',
    'ansatte',
    'aksjonaerer_antall_end',
    'aksjonaerer_antall_beginning',
    'aksjonaerer_mean_end',
    'aksjonaerer_std_end',
    'aksjonaerer_max_end',
    'aksjonaerer_min_end',
    'aksjonaerer_mean_beginning',
    'aksjonaerer_std_beginning',
    'aksjonaerer_max_beginning',
    'aksjonaerer_min_beginning',
    'antall_styremedlemmer_ikke_aksjonaer',
    'andel_styremedlemmer_ikke_aksjonaer',
    'aksjonaerer_andel_fylkesnr',
]

# CATEGORICAL variables with missing values to be imputed
var_mis_cat = [
    'dummy_samme_dl_sl', 
    'dummy_dl_in_styre',
    'dummy_to_dl',
    'dummy_dl_kjonn_K',
    'dummy_sl_kjonn_K',
    'dummy_dl_fylkesnr',
    'dummy_sl_fylkesnr',
    'fravalg_revisjon',
    'bistand_regnskapsforer',
    'utarbeidet_regnskapsforer',
]

print('-------------------------')
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data20[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data20[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data20[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data20[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data20[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data20[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data20[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data20[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data20[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data20[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data20[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data20[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data20[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data20[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data20[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data20[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data20[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data20[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data20[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data20[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data20[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data20[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data20[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data20[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data20[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data20[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data20[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data20[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data20[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data20[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data20[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data20[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data20[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' BEFORE imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data20[var_mis_cat[9]]))))
print('-------------------------')



# variables used in the vector space for the kNN algorithm for imputation 
var_use = [
    'SUM EIENDELER',
    'sum_inntekter',
    'lonnskostnad',
    'age_in_days',
]
data20 = KNNImputer_rrw(var_mis_con,var_mis_cat,var_use,n_neighbors_con,data20,do_standardize_vector_space)

print('-------------------------')
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[0],np.sum(pd.isnull(data20[var_mis_con[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[1],np.sum(pd.isnull(data20[var_mis_con[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[2],np.sum(pd.isnull(data20[var_mis_con[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[3],np.sum(pd.isnull(data20[var_mis_con[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[4],np.sum(pd.isnull(data20[var_mis_con[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[5],np.sum(pd.isnull(data20[var_mis_con[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[6],np.sum(pd.isnull(data20[var_mis_con[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[7],np.sum(pd.isnull(data20[var_mis_con[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[8],np.sum(pd.isnull(data20[var_mis_con[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[9],np.sum(pd.isnull(data20[var_mis_con[9]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[10],np.sum(pd.isnull(data20[var_mis_con[10]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[11],np.sum(pd.isnull(data20[var_mis_con[11]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[12],np.sum(pd.isnull(data20[var_mis_con[12]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[13],np.sum(pd.isnull(data20[var_mis_con[13]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[14],np.sum(pd.isnull(data20[var_mis_con[14]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[15],np.sum(pd.isnull(data20[var_mis_con[15]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[16],np.sum(pd.isnull(data20[var_mis_con[16]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[17],np.sum(pd.isnull(data20[var_mis_con[17]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[18],np.sum(pd.isnull(data20[var_mis_con[18]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[19],np.sum(pd.isnull(data20[var_mis_con[19]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[20],np.sum(pd.isnull(data20[var_mis_con[20]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[21],np.sum(pd.isnull(data20[var_mis_con[21]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[22],np.sum(pd.isnull(data20[var_mis_con[22]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_con[23],np.sum(pd.isnull(data20[var_mis_con[23]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[0],np.sum(pd.isnull(data20[var_mis_cat[0]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[1],np.sum(pd.isnull(data20[var_mis_cat[1]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[2],np.sum(pd.isnull(data20[var_mis_cat[2]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[3],np.sum(pd.isnull(data20[var_mis_cat[3]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[4],np.sum(pd.isnull(data20[var_mis_cat[4]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[5],np.sum(pd.isnull(data20[var_mis_cat[5]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[6],np.sum(pd.isnull(data20[var_mis_cat[6]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[7],np.sum(pd.isnull(data20[var_mis_cat[7]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[8],np.sum(pd.isnull(data20[var_mis_cat[8]]))))
print('Number of missing values for variable \'{}\' AFTER imputing: {}'\
    .format(var_mis_cat[9],np.sum(pd.isnull(data20[var_mis_cat[9]]))))
print('-------------------------')


data06.to_csv('nadata06.csv', index = False)
data07.to_csv('nadata07.csv', index = False)
data08.to_csv('nadata08.csv', index = False)
data09.to_csv('nadata09.csv', index = False)
data10.to_csv('nadata10.csv', index = False)
data11.to_csv('nadata11.csv', index = False)
data12.to_csv('nadata12.csv', index = False)
data13.to_csv('nadata13.csv', index = False)
data14.to_csv('nadata14.csv', index = False)
data15.to_csv('nadata15.csv', index = False)
data16.to_csv('nadata16.csv', index = False)
data17.to_csv('nadata17.csv', index = False)
data18.to_csv('nadata18.csv', index = False)
data19.to_csv('nadata19.csv', index = False)
data20.to_csv('nadata20.csv', index = False)


nadata_uten_missing = data06.append(data07)
nadata_uten_missing = nadata_uten_missing.append(data08)
nadata_uten_missing = nadata_uten_missing.append(data09)
nadata_uten_missing = nadata_uten_missing.append(data10)
nadata_uten_missing = nadata_uten_missing.append(data11)
nadata_uten_missing = nadata_uten_missing.append(data12)
nadata_uten_missing = nadata_uten_missing.append(data13)
nadata_uten_missing = nadata_uten_missing.append(data14)
nadata_uten_missing = nadata_uten_missing.append(data15)
nadata_uten_missing = nadata_uten_missing.append(data16)
nadata_uten_missing = nadata_uten_missing.append(data17)
nadata_uten_missing = nadata_uten_missing.append(data18)
nadata_uten_missing = nadata_uten_missing.append(data19)
nadata_uten_missing = nadata_uten_missing.append(data20)

nadata_uten_missing.to_csv('nadata_uten_missing.csv', index=False)
