import numpy as np
# Copyright © 2005-2019, NumPy Developers.

import pandas as pd
# Copyright (c) 2008-2012, AQR Capital Management, LLC, Lambda Foundry, Inc. and PyData Development Team

import os

##################################################################
##  Load data brreg                                             ##
##################################################################
print('-----------------------------------------')
print('Loading data:')
print('-----------------------------------------')
folder_name = '../../datasett_aarsregnskaper/data5/'
files = os.listdir(folder_name)
for current_file in files:
    file_year = int(current_file[0:4])

    # LOADING DATA
    data_loaded = pd.read_csv(folder_name+current_file,sep=';',low_memory=False)

    # Adding all data together into data
    if current_file == files[0]:
        data_brreg = pd.DataFrame(columns=data_loaded.columns)
    data_brreg = pd.concat([data_brreg,data_loaded])
    print('Imported for accounting year {}'.format(file_year))

# Reset index 
data_brreg = data_brreg.reset_index(drop=True)

# Double checking if all is unique
def warning_if_not_all_rows_unique(data_brreg):
    unique_orgnr = data_brreg.groupby(['orgnr','regnaar']).size().reset_index()
    temp = unique_orgnr[0].unique()
    if len(temp)==1:
        print('All orgnr unique')
    else:
        print('WARNING: not all orgnr unique')

warning_if_not_all_rows_unique(data_brreg)

# Removing unused objects
del data_loaded

# Filtering
# For many rows, accounting variables have no value. No value for accounting
# variables actually means that it is zero. Thus, use .fillna(0).
ind = (data_brreg['sum_omsetning_EUR'].fillna(0)<=50e6) | (data_brreg['sum_eiendeler_EUR'].fillna(0)<=43e6)
ind = ind & (data_brreg['regnaar']<=2020)
ind = ind & (data_brreg['orgform']=='AS')
data_brreg_filtered = data_brreg[ind]
data_brreg_filtered = data_brreg_filtered.reset_index(drop=True) # Reset index


##################################################################
##  Load data enin                                              ##
##################################################################

data_accounting = pd.read_csv('accounting_all.csv',sep=',',low_memory=False)

# data_accounting = pd.read_csv('accounting.csv',sep=',',low_memory=False)

# data_accounting_eur = pd.read_csv('accounting_eur.csv',sep=',',low_memory=False)

# # Checking if same
# if data_accounting.shape[0]!=data_accounting_eur.shape[0]:
#     print('ERROR: not same')
# if np.sum(data_accounting['company.org_nr'])!=np.sum(data_accounting_eur['company.org_nr']):
#     print('ERROR: not same')
# if np.sum(data_accounting['accounts.accounting_year'])!=np.sum(data_accounting_eur['accounts.accounting_year']):
#     print('ERROR: not same')

##################################################################
##  Merging                                                     ##
##################################################################

# Renaming columns
data_accounting.rename(columns = {'accounts.accounting_year':'regnaar', 'company.org_nr':'orgnr'}, inplace = True)

# Making columns to same type
data_brreg_filtered['orgnr'] = data_brreg_filtered['orgnr'].astype(int)
data_brreg_filtered['regnaar'] = data_brreg_filtered['regnaar'].astype(int)
data_accounting['orgnr'] = data_accounting['orgnr'].astype(int)
data_accounting['regnaar'] = data_accounting['regnaar'].astype(int)

# Checking that none of the keys are missing
if np.sum(pd.isnull(data_brreg_filtered['orgnr']))!=0:
    print('ERROR')
if np.sum(pd.isnull(data_accounting['regnaar']))!=0:
    print('ERROR')
if np.sum(pd.isnull(data_brreg_filtered['orgnr']))!=0:
    print('ERROR')
if np.sum(pd.isnull(data_accounting['regnaar']))!=0:
    print('ERROR')

# Merging. All observations from Brreg is included, and observations from Enin that also is in Brreg included
# Observations (financial statements) only in Enin is excluded
data_brreg_filtered = data_brreg_filtered.reset_index(drop=True) # Reset index
data_accounting = data_accounting.reset_index(drop=True) # Reset index
data_merged = pd.merge(data_brreg_filtered,data_accounting,\
    on=['orgnr','regnaar'],\
    how='left')

# Checking who is missing
unique_aarsregnskaper = data_merged.groupby(['orgnr','regnaar']).size().reset_index()
num_unique_aarsregnskaper = unique_aarsregnskaper.shape[0]
print('Aarsregnskaper not in data from Enin:')
aarsregnskaper_missing_in_enin = unique_aarsregnskaper[unique_aarsregnskaper[0]==1][['orgnr','regnaar']]
print(aarsregnskaper_missing_in_enin)

# Removing double observations
data_merged_after_dropping_double=data_merged.drop_duplicates(subset=['orgnr','regnaar'])

if data_merged_after_dropping_double.shape[0]!=num_unique_aarsregnskaper:
    print('ERROR')
if data_merged_after_dropping_double.shape[0]!=data_brreg_filtered.shape[0]:
    print('ERROR')


if False:
    # Checking
    if np.sum(pd.isnull(data_accounting['company.org_nr']))!=0:
        print('ERROR')

    data_difference = data_merged[pd.isnull(data_merged['company.org_nr'])]
    print('Percent difference: {}'.format(100*data_difference.shape[0]/data_merged.shape[0]))

    regnaar_dif = 2016
    data_difference[data_difference.regnaar==regnaar_dif][[
        'orgnr',
        'regnaar',
        'orgform',
        'Sum inntekter',
        'SUM EIENDELER',
        'sum_omsetning_EUR',
        'sum_eiendeler_EUR']].iloc[0:1000].to_excel('data_difference_'+regnaar_dif+'.xlsx',index=False)

    data_difference[[
        'orgnr',
        'regnaar',
        'orgform',
        'Sum inntekter',
        'SUM EIENDELER',
        'sum_omsetning_EUR',
        'sum_eiendeler_EUR']].to_excel('data_difference_all.xlsx',index=False)

    pd.pivot_table(data_difference,\
        values='orgnr',\
        index=['regnaar'],\
        # columns=['orgform'],\
        aggfunc='count')

    # Checking
    if np.sum(pd.isnull(data_brreg_filtered['avslutningsdato']))!=0:
        print('ERROR')

    data_difference = data_merged[pd.isnull(data_merged['avslutningsdato'])]
    print('Percent difference: {}'.format(100*data_difference.shape[0]/data_merged.shape[0]))

    ind = data_merged['organization_type.organization_type_code']!='AS'
    ind = ind & (pd.isnull(data_merged['organization_type.organization_type_code'])==False)
    temp = data_merged[ind]

    temp[[
        'orgnr',
        'regnaar',
        'orgform',
        'organization_type.organization_type_code']]

# Extracting only variables used
columns = data_accounting.columns.tolist()
data_for_saving = data_merged_after_dropping_double[columns]
data_for_saving.rename(columns = {'regnaar':'accounts.accounting_year', 'orgnr':'company.org_nr'}, inplace = True)

# Checking
if data_for_saving.shape[0]!=data_brreg_filtered.shape[0]:
    print('ERROR')
if data_for_saving.shape[1]!=data_accounting.shape[1]:
    print('ERROR')


data_for_saving.to_csv('accounting_from_ranik.csv', sep=',', index=False) 