library(glmnet)
library(readr)
library(pROC)
library(RColorBrewer)
library(DescTools)
library(car)

# lager tomme lister for lagring av resultater
coef_list = list()
rekkefolge_list = list()
auc_train_list = list()
auc_test_list = list()
summary_lrmod = list()
vif = list()
sum_auc_test = 0
sum_auc_train = 0
brier_score_train = 0
brier_score_test = 0


for (year in 2010:2020){
  
  #laste inn data
  X_train = read_csv(paste("#sett_inn_filnavn#",year,".csv", sep="", collapse=NULL))[-1]
  y_train = read_csv(paste("#sett_inn_filnavn#",year,".csv", sep="", collapse=NULL))[-1]
  X_test = read_csv(paste("#sett_inn_filnavn#",year,".csv", sep="", collapse=NULL))[-1]
  y_test = read_csv(paste("#sett_inn_filnavn#",year,".csv", sep="", collapse=NULL))[-1]
  
  #gjøre om til matrise
  X_train = as.matrix(X_train)
  X_test = as.matrix(X_test)
  
  #utfører 3-fold kryssvalidering med cv.glmnet basert på kriteriet AUC, for 100 verdier av lambda (default)
  set.seed(1) #setter seed for kryssvalidering 
  cvfit_train = cv.glmnet(X_train, as.numeric(y_train$bankrupt_fs), alpha=1, family = "binomial", nfolds = 3, type.measure = "auc")
  times = par(family ='serif') #endrer font
  lambda_plot = log(cvfit_train$lambda.min) - 0.5
  par(mar=c(5,5,3,15))
  plot(cvfit_train, xlim=c(lambda_plot, -3.5), ylim=c(0.6,0.9), cex.lab=1.5, cex.axis = 1.3) #plot in-sample auc
  auc_plot = recordPlot() 
  assign(paste0('auc_plot', year), auc_plot)
  
  #lager oversikt over utvalgte variabler
  w=as.matrix(coef(cvfit_train))
  keep_x = rownames(w)[w!=0]
  keep_x = keep_x[!keep_x == '(Intercept)']
  x1 = X_train[,keep_x]
  
  #lager input for LR-modell med y og de utvalgte variablene
  ytrain = y_train$bankrupt_fs
  selected_x_train = paste(keep_x, sep=" ", " + ", collapse = " ")
  selected_train = paste("ytrain ~", selected_x_train)
  selected_train = substr(selected_train, 1, nchar(selected_train)-2)

  #lager LR-modell med de utvalgte variablene
  best_lambda = cvfit_train$lambda.1se
  data_train = cbind(X_train, y_train$bankrupt_fs)
  lrmod = glm(paste(selected_train), data=data.frame(data_train), family='binomial')
  coef_lrmod = coef(lrmod)
  coef_list[[year]] = coef_lrmod
  summary_lrmod[[year]] = summary(lrmod)
  vif[[year]] = vif(lrmod) 
  
  #sjekker tilpasning på treningssettet
  y_train$probs = predict(lrmod, data.frame(X_train), type = 'response')
  roc_obj_train = roc(as.numeric(y_train$bankrupt_fs), as.numeric(y_train$probs))
  sum_auc_train = sum_auc_train+roc_obj_train$auc
  auc_train = roc_obj_train$auc
  auc_train_list[[year]] = auc_train
  roc_obj_train = recordPlot() 
  assign(paste0('roc_obj_train', year), roc_obj_train)
  brier_score_train[[year]] = BrierScore(y_train$probs, y_train$bankrupt_fs)
  
  #sjekker tilpasning på valideringssettet
  y_test$probs = predict(lrmod, data.frame(X_test), type = 'response') 
  roc_obj_test = roc(as.numeric(y_test$bankrupt_fs), as.numeric(y_test$probs))
  sum_auc_test = sum_auc_test+roc_obj_test$auc
  auc_test = roc_obj_test$auc
  auc_test_list[[year]] = auc_test
  roc_obj_test = recordPlot() 
  assign(paste0('roc_obj_test', year), roc_obj_test)
  brier_score_test[[year]] = BrierScore(y_test$probs,y_test$bankrupt_fs)
  
  # lage plot av lasso path og henter ut oversikt over rekkefølgen variblene blir valgt ut
  grid = 10^seq(10,-10, length = 100)
  lasso_path = glmnet(x1, as.numeric(unlist(y_train$bankrupt_fs)), alpha = 1, lambda = grid, family = 'binomial', standardize = TRUE)
  betas <- as.matrix(lasso_path$beta)
  varlist <- row.names(betas)
  which_step <- rep(NA, length(varlist))
  for(i in 1:length(varlist))
  {
    which_step[i] <- which.max(betas[i,] != 0)
  }
  i_rekkefolge = varlist[order(which_step)]
  rekkefolge_list[[year]] = i_rekkefolge
  
  #fargepalett for bruk i lasso-plot
  colors = c("dodgerblue2", "#E31A1C", "green4", "#6A3D9A", "#FF7F00", "black", "gold1", "skyblue2", "#FB9A99", "palegreen2", "#CAB2D6", "#FDBF6F", "gray70", "khaki2", "maroon", "orchid1", "deeppink1", "blue1", "steelblue4", "darkturquoise", "green1", "yellow4", "yellow3", "darkorange4")
  
  #lager lasso-plot
  times = par(family ='serif')
  par(mar=c(5,5,3,15))
  plot(lasso_path, 'lambda', col = colors, cex.lab=1.5, cex.axis = 1.3, lwd = 2, xlab = 'Log(λ)', 
       ylab = 'Standardiserte koeffisienter', xlim=c(lambda_plot, -3.5),
       ylim=c((min(coef_lrmod[-1])), max(coef_lrmod[-1])))
  legend(x='topright',
         inset = c(-0.75,0), # justeres avhengig av størrelse
         title = expression(bold("Variabler valgt av LASSO")), 
         legend = colnames(x1), 
         col = colors,
         lwd = 2, 
         cex = 1.1,
         xpd = TRUE)
  abline(v=log(best_lambda), col='black', lwd = 2, lty = 'dashed')
  abline(v=log(cvfit_train$lambda.min), col = 'black', lwd = 1.25, lty = 'dashed')
  my_plot = recordPlot()
  assign(paste0('plot', year), my_plot)
}

#hente ut resultater
for (year in 2010:2020){
  print(summary_lrmod[year])
  print(vif[year])
  print(brier_score_train[year])
  print(brier_score_test[year])
  print(auc_train_list[year])
  print(auc_test_list[year])
  print(rekkefolge_list[year])
}

