import numpy as np
import pandas as pd
import time 

## Laster inn data
data = pd.read_csv("#filnavn#.csv")

temp_num_before = data.shape[0]
data = data[pd.isnull(data['accounts.accounting_to_date'])==False]
data = data.reset_index(drop=True)
print('Antall årsregnskaper fjernet på grunn av manglende verdier: {}'.format(temp_num_before - data.shape[0]))

unique_aarsregnskaper = data.groupby(['accounts.accounting_year','company.org_nr']).size().reset_index()
num_unique_aarsregnskaper = unique_aarsregnskaper.shape[0]
non_unique_aarsregnskap = unique_aarsregnskaper[unique_aarsregnskaper[0]>1][['accounts.accounting_year','company.org_nr']]

# fjerner doble dersom de er identiske
def extracting_only_month_and_day(x):
    return x[-5:]

for i in range(non_unique_aarsregnskap.shape[0]):
    temp = non_unique_aarsregnskap.iloc[i]
    temp = data[(data['company.org_nr']==temp['company.org_nr'])&(data['accounts.accounting_year']==temp['accounts.accounting_year'])]
    ind = temp.iloc[0]['accounts.accounting_from_date']==temp.iloc[1]['accounts.accounting_from_date']
    ind = ind & (temp.iloc[0]['accounts.accounting_from_date']==temp.iloc[1]['accounts.accounting_from_date'])
    ind = ind & (temp.iloc[0]['accounts.accounting_to_date']==temp.iloc[1]['accounts.accounting_to_date'])
    ind = ind & (temp.iloc[0]['company_details.employees']==temp.iloc[1]['company_details.employees'])
    ind = ind & (temp.iloc[0]['company_details.business_address_municipality_number']==temp.iloc[1]['company_details.business_address_municipality_number'])
    if ind:
        data = data.drop(temp.index[0]) # fjerner kun 1 av observasjonene
    else:
        temp_mont_day = temp['accounts.accounting_to_date'].apply(extracting_only_month_and_day)
        data = data.drop(temp[temp_mont_day!='12-31'].index[0])

# reset indeks
data = data.reset_index(drop=True)

# gir en feilmelding dersom samme årsregnskap er på flere rader
unique_aarsregnskaper = data.groupby(['accounts.accounting_year','company.org_nr']).size().reset_index()
temp = unique_aarsregnskaper[0].unique()
if len(temp)!=1:
    print('WARNING: not all aarsregnskaper unique')
if num_unique_aarsregnskaper!=unique_aarsregnskaper.shape[0]:
    print('ERROR in handling double entries of aarsregnskaper')

df_dl       = pd.read_csv("chief.csv")
df_sl       = pd.read_csv("board_chairman.csv")
df_styre    = pd.read_csv("board.csv")
df_aks      = pd.read_csv("shareholder.csv")

# Endrer navn på kolonnen for orgnr for lettere oversikt
data            = data.rename(columns={'company.org_nr':'orgnr'})
df_dl           = df_dl.rename(columns={'company.org_nr':'orgnr'})
df_sl           = df_sl.rename(columns={'company.org_nr':'orgnr'})
df_styre        = df_styre.rename(columns={'company.org_nr':'orgnr'})
df_aks          = df_aks.rename(columns={'share_issuer_company.org_nr':'orgnr'})

# Endrer navn på kolonnen for kommunenr for lettere oversikt
data            = data.rename(columns={'company_details.business_address_municipality_number':'kommunenr'})
df_dl           = df_dl.rename(columns={'person.municipality_code':'kommunenr'})
df_sl           = df_sl.rename(columns={'person.municipality_code':'kommunenr'})
df_styre        = df_styre.rename(columns={'person.municipality_code':'kommunenr'})
df_aks          = df_aks.rename(columns={'shareholder_person.municipality_code':'kommunenr'})


## endrer dato fra 1000-01-01 til 1900-01-01 
df_dl.loc[(df_dl['person_company_role.from_date']=='1000-01-01'), 'person_company_role.from_date'] = '1900-01-01'
df_sl.loc[(df_sl['person_company_role.from_date']=='1000-01-01'), 'person_company_role.from_date'] = '1900-01-01'
df_styre.loc[(df_styre['person_company_role.from_date']=='1000-01-01'), 'person_company_role.from_date'] = '1900-01-01'


# Gjør så kolonner blir på tidsformat
df_dl['person_company_role.to_date'] =  pd.to_datetime(df_dl['person_company_role.to_date'])
df_dl['person_company_role.from_date'] =  pd.to_datetime(df_dl['person_company_role.from_date'])
df_sl['person_company_role.to_date'] =  pd.to_datetime(df_sl['person_company_role.to_date'])
df_sl['person_company_role.from_date'] =  pd.to_datetime(df_sl['person_company_role.from_date'])
df_styre['person_company_role.to_date'] =  pd.to_datetime(df_styre['person_company_role.to_date'])
df_styre['person_company_role.from_date'] =  pd.to_datetime(df_styre['person_company_role.from_date'])

# Manglende til-dato betyr at styre/styreleder/daglig leder fortsatt har rollen. Setter derfor til-dato til en veldig høy dato:
df_dl.loc[pd.isnull(df_dl['person_company_role.to_date']),'person_company_role.to_date']        = pd.to_datetime('2159-06-16')
df_sl.loc[pd.isnull(df_sl['person_company_role.to_date']),'person_company_role.to_date']        = pd.to_datetime('2159-06-16')
df_styre.loc[pd.isnull(df_styre['person_company_role.to_date']),'person_company_role.to_date']  = pd.to_datetime('2159-06-16')


# Gjør om til kjønn 'K' (kvinne) og 'M' (mann)
df_dl.loc[(df_dl['person.gender_uuid']=='a091eaef-e659-4920-9bdc-d42a75aab765'),'person.gender_uuid'] = 'M'
df_dl.loc[(df_dl['person.gender_uuid']=='27ef7f8e-c23d-424b-8050-667a750b1574'),'person.gender_uuid'] = 'K'
df_sl.loc[(df_sl['person.gender_uuid']=='a091eaef-e659-4920-9bdc-d42a75aab765'),'person.gender_uuid'] = 'M'
df_sl.loc[(df_sl['person.gender_uuid']=='27ef7f8e-c23d-424b-8050-667a750b1574'),'person.gender_uuid'] = 'K'
df_styre.loc[(df_styre['person.gender_uuid']=='a091eaef-e659-4920-9bdc-d42a75aab765'),'person.gender_uuid'] = 'M'
df_styre.loc[(df_styre['person.gender_uuid']=='27ef7f8e-c23d-424b-8050-667a750b1574'),'person.gender_uuid'] = 'K'

# Sjekker at det ikke er flere verdier for kjønn
if len(df_dl['person.gender_uuid'].unique())>3:
    print('Flere verdier for kjønn for daglig leder')
if len(df_sl['person.gender_uuid'].unique())>3:
    print('Flere verdier for kjønn for styreleder')
if len(df_styre['person.gender_uuid'].unique())>3:
    print('Flere verdier for kjønn for styreleder')

# Sjekker at det ikke er manglende verdier for fra-dato:
if np.sum(pd.isnull(df_dl['person_company_role.from_date']))!=0:
    print('ERROR: missing values from date dl')
if np.sum(pd.isnull(df_sl['person_company_role.from_date']))!=0:
    print('ERROR: missing values from date sl')
if np.sum(pd.isnull(df_styre['person_company_role.from_date']))!=0:
    print('ERROR: missing values from date sl')


## Lager variabler

def funksjon_for_identifisere_styre_dl_sl(df,orgnr,balance_date):
    return df[df['orgnr']==orgnr][(df[df['orgnr']==orgnr]['person_company_role.from_date']<balance_date)&(df[df['orgnr']==orgnr]['person_company_role.to_date']>balance_date)]



def check_kom_fyl(var1,var2):
    if pd.isnull(var1)|pd.isnull(var2):
        return None
    else:
        return var1==var2

def check_styre_aksj(var1,var2):
    if len(var1)==0:
        return None
    elif len(var1)>1:
        if (np.sum(pd.isnull(var1.tolist()))!=0)|pd.isnull(var2):
            return None
        else:
            return np.mean(var1==var2)
    else: # len(var1)==0
        var1 = var1.iloc[0]
        if pd.isnull(var1)|pd.isnull(var2):
            return None
        else:
            return np.mean(var1==var2)

# Genererer fylkesnummer
def func_fylkesnr(x):
    if pd.isnull(x)==False:
        return int(np.floor(np.round(x)/100))
    else:
        return None

df_dl['fylkesnr']       = df_dl['kommunenr'].apply(func_fylkesnr)
df_sl['fylkesnr']       = df_sl['kommunenr'].apply(func_fylkesnr)
data['fylkesnr']        = data['kommunenr'].apply(func_fylkesnr)
df_styre['fylkesnr']    = df_styre['kommunenr'].apply(func_fylkesnr)
df_aks['fylkesnr']      = df_aks['kommunenr'].apply(func_fylkesnr)

# Lager tomme lister for å fylle inn variabelverdier:
antall_aarsregnskaper = data.shape[0]

alder_dl            = [None]*antall_aarsregnskaper
alder_sl            = [None]*antall_aarsregnskaper
dummy_samme_dl_sl   = [None]*antall_aarsregnskaper
dummy_dl_in_styre   = [None]*antall_aarsregnskaper
dummy_dl_kjonn_K    = [None]*antall_aarsregnskaper
dummy_sl_kjonn_K    = [None]*antall_aarsregnskaper
dummy_to_dl         = [None]*antall_aarsregnskaper

dummy_sl_kommunenr  = [None]*antall_aarsregnskaper
dummy_dl_kommunenr  = [None]*antall_aarsregnskaper

dummy_dl_fylkesnr   = [None]*antall_aarsregnskaper
dummy_sl_fylkesnr   = [None]*antall_aarsregnskaper

styre_alder_mean    = [None]*antall_aarsregnskaper
styre_alder_std     = [None]*antall_aarsregnskaper
styre_antall        = [None]*antall_aarsregnskaper
styre_andel_kjonn_K = [None]*antall_aarsregnskaper

styre_andel_kommunenr       = [None]*antall_aarsregnskaper
aksjonaerer_andel_kommunenr = [None]*antall_aarsregnskaper

styre_andel_fylkesnr        = [None]*antall_aarsregnskaper
aksjonaerer_andel_fylkesnr  = [None]*antall_aarsregnskaper

aksjonaerer_antall_end          = [None]*antall_aarsregnskaper
aksjonaerer_antall_beginning    = [None]*antall_aarsregnskaper

aksjonaerer_mean_end    = [None]*antall_aarsregnskaper
aksjonaerer_std_end     = [None]*antall_aarsregnskaper
aksjonaerer_max_end     = [None]*antall_aarsregnskaper
aksjonaerer_min_end     = [None]*antall_aarsregnskaper

aksjonaerer_mean_beginning    = [None]*antall_aarsregnskaper
aksjonaerer_std_beginning     = [None]*antall_aarsregnskaper
aksjonaerer_max_beginning     = [None]*antall_aarsregnskaper
aksjonaerer_min_beginning    = [None]*antall_aarsregnskaper

sl_share_aksje      = [None]*antall_aarsregnskaper
dl_share_aksje      = [None]*antall_aarsregnskaper
styre_share_aksje   = [None]*antall_aarsregnskaper

antall_styremedlemmer_ikke_aksjonaer    = [None]*antall_aarsregnskaper
andel_styremedlemmer_ikke_aksjonaer     = [None]*antall_aarsregnskaper

fnr_dl = [None]*antall_aarsregnskaper
fnr_sl = [None]*antall_aarsregnskaper

antall_doble_daglig_leder = 0
antall_aarsregnskaper_med_to_forskjellige_styreledere = 0
antall_ingen_styre = 0

# Kjører for-løkke som går gjennom alle årsregnskaper og lager variabler:
t_start = time.time()
for i in range(antall_aarsregnskaper):
    orgnr = data.iloc[i]['orgnr']
    accounting_year = data.iloc[i]['accounts.accounting_year']
    balance_date = data.iloc[i]['accounts.accounting_to_date']
    kommunenr = data.iloc[i]['kommunenr']
    fylkesnr = data.iloc[i]['fylkesnr']

    # Identifiserer daglig leder, styreleder og styre ved balansedato:
    dl                      = funksjon_for_identifisere_styre_dl_sl(df_dl,orgnr,balance_date)
    sl                      = funksjon_for_identifisere_styre_dl_sl(df_sl,orgnr,balance_date) 
    styre                   = pd.concat([funksjon_for_identifisere_styre_dl_sl(df_styre,orgnr,balance_date),sl])# Legger styreleder inn i styret
    aksjonaerer_end         = df_aks[(df_aks['orgnr']==orgnr)&(df_aks['company_share_ownership.year']==accounting_year)]    
    aksjonaerer_beginning   = df_aks[(df_aks['orgnr']==orgnr)&(df_aks['company_share_ownership.year']==(accounting_year-1))]    

    check_registered_aksjonaerer_end = aksjonaerer_end.shape[0]>0

    # Sjekker om samme daglig leder og styreleder
    if (dl.shape[0]>0)&(sl.shape[0]>0):
        # Gjør kun det nedenfor dersom det er registrert minst
        # én daglig leder og minst én styreleder. Dersom
        # kun en av disse mangles så forblir dummy_samme_dl_sl=None

        # setter dummy_samme_dl_sl til True dersom
        # minst én person er samme i både styreleder og daglig leder
        # ellers til False
        temp=False
        for jj in dl['person_company_role.person_uuid']:
            temp = temp | (jj in sl['person_company_role.person_uuid'].values)
        dummy_samme_dl_sl[i] = temp

    # Variabler daglig leder
    if dl.shape[0]==1: # Lager kun variabel dersom det er én identifisert person
        dummy_to_dl[i] = 0
        alder_dl[i] = accounting_year - dl['person.birth_year'].iloc[0]
        dummy_dl_kjonn_K[i] = dl['person.gender_uuid'].iloc[0]=='K'
        if check_registered_aksjonaerer_end:
            dl_share_aksje[i] = np.sum(aksjonaerer_end[aksjonaerer_end['company_share_ownership.shareholder_entity_uuid']==dl['person_company_role.person_uuid'].iloc[0]]['company_share_ownership.ownership'])
        dummy_dl_kommunenr[i]   = check_kom_fyl(dl['kommunenr'].iloc[0],kommunenr)
        dummy_dl_fylkesnr[i]    = check_kom_fyl(dl['fylkesnr'].iloc[0],fylkesnr)
        fnr_dl[i]               = dl['fylkesnr']
    elif dl.shape[0]==2: # Dersom to daglige ledere, behold kun den ene
        dummy_to_dl[i] = 1
        antall_doble_daglig_leder += 1
        dl = dl.iloc[0]
        alder_dl[i] = accounting_year - dl['person.birth_year']
        dummy_dl_kjonn_K[i] = dl['person.gender_uuid']=='K'
        if check_registered_aksjonaerer_end:
            dl_share_aksje[i] = np.sum(aksjonaerer_end[aksjonaerer_end['company_share_ownership.shareholder_entity_uuid']==dl['person_company_role.person_uuid']]['company_share_ownership.ownership'])
        dummy_dl_kommunenr[i]   = check_kom_fyl(dl['kommunenr'],kommunenr)
        dummy_dl_fylkesnr[i]    = check_kom_fyl(dl['fylkesnr'],fylkesnr)
        fnr_dl[i]               = dl['fylkesnr']
    elif dl.shape[0]>2: # Gir feilmelding om det er flere enn to personer
        print('ERROR: more than two dl for orgnr {} and accounting_year {}'.format(orgnr,accounting_year))

    # Variabler styreleder
    if sl.shape[0]==1: # Lager kun variabel dersom det er én identifisert person
        alder_sl[i] = accounting_year - sl['person.birth_year'].iloc[0]
        dummy_sl_kjonn_K[i] = sl['person.gender_uuid'].iloc[0]=='K'
        if check_registered_aksjonaerer_end:
            sl_share_aksje[i] = np.sum(aksjonaerer_end[aksjonaerer_end['company_share_ownership.shareholder_entity_uuid']==sl['person_company_role.person_uuid'].iloc[0]]['company_share_ownership.ownership'])
        dummy_sl_kommunenr[i]   = check_kom_fyl(sl['kommunenr'].iloc[0],kommunenr)
        dummy_sl_fylkesnr[i]    = check_kom_fyl(sl['fylkesnr'].iloc[0],fylkesnr)
        fnr_sl[i]               = sl['fylkesnr']
    elif sl.shape[0]==2:
        if (sl['person_company_role.from_date'].iloc[0]==sl['person_company_role.from_date'].iloc[1])&(sl['person_company_role.to_date'].iloc[0]==sl['person_company_role.to_date'].iloc[1])&(sl['person.gender_uuid'].iloc[0]==sl['person.gender_uuid'].iloc[1]):
            sl = sl[np.sum(pd.isnull(sl),1)==np.min(np.sum(pd.isnull(sl),1))] # beholder den med minst missing
            alder_sl[i] = accounting_year - sl['person.birth_year'].iloc[0]
            dummy_sl_kjonn_K[i] = sl['person.gender_uuid'].iloc[0]=='K'
            if check_registered_aksjonaerer_end:
                sl_share_aksje[i] = np.sum(aksjonaerer_end[aksjonaerer_end['company_share_ownership.shareholder_entity_uuid']==sl['person_company_role.person_uuid'].iloc[0]]['company_share_ownership.ownership'])
            dummy_sl_kommunenr[i]   = check_kom_fyl(sl['kommunenr'].iloc[0],kommunenr)
            dummy_sl_fylkesnr[i]    = check_kom_fyl(sl['fylkesnr'].iloc[0],fylkesnr)
            fnr_sl[i]               = sl['fylkesnr']
        else:
            print('ERROR: two styreleder, but seems to be different persons. orgnr {} and accounting_year {}'.format(orgnr,accounting_year))
            antall_aarsregnskaper_med_to_forskjellige_styreledere += 1        
    elif sl.shape[0]>2:
        print('ERROR: more than two styreleder for orgnr {} and accounting_year {}'.format(orgnr,accounting_year))
    
    # Variabler styre
    if styre.shape[0]>0:
        styre_antall[i]         = styre.shape[0]

        ind_aksjonaerer_styremedlem = [False]*aksjonaerer_end.shape[0]
        for j in range(styre_antall[i]):
            ind_aksjonaerer_styremedlem = ind_aksjonaerer_styremedlem | (aksjonaerer_end['company_share_ownership.shareholder_entity_uuid']==styre['person_company_role.person_uuid'].iloc[j])
        if check_registered_aksjonaerer_end:
            styre_share_aksje[i] = np.sum(aksjonaerer_end[ind_aksjonaerer_styremedlem]['company_share_ownership.ownership'])    
            antall_styremedlemmer_ikke_aksjonaer[i] = styre_antall[i]-np.sum(ind_aksjonaerer_styremedlem)
            andel_styremedlemmer_ikke_aksjonaer[i]  = antall_styremedlemmer_ikke_aksjonaer[i]/styre_antall[i]

        styre_andel_kommunenr[i]   = check_styre_aksj(styre['kommunenr'],kommunenr)
        styre_andel_fylkesnr[i]    = check_styre_aksj(styre['fylkesnr'],fylkesnr)

        # Variabler styre
        styre_alder_mean[i]     = np.mean(accounting_year - styre['person.birth_year'])
        styre_alder_std[i]      = np.std(accounting_year - styre['person.birth_year'])

        styre_andel_kjonn_K[i] = np.mean(styre['person.gender_uuid']=='K')

        if dl.shape[0]==1:
            dummy_dl_in_styre[i] = dl['person_company_role.person_uuid'].iloc[0] in styre['person_company_role.person_uuid'].values
    else:
        antall_ingen_styre += 1
    
    # Variabler aksjonærer
    if aksjonaerer_end.shape[0]>0:
        aksjonaerer_antall_end[i]       = aksjonaerer_end.shape[0]
    if aksjonaerer_beginning.shape[0]>0:
        aksjonaerer_antall_beginning[i] = aksjonaerer_beginning.shape[0]

    temp = aksjonaerer_end['company_share_ownership.ownership']
    aksjonaerer_mean_end[i]        = np.mean(temp)
    aksjonaerer_std_end[i]         = np.std(temp)
    aksjonaerer_max_end[i]         = np.max(temp)
    aksjonaerer_min_end[i]         = np.min(temp)

    temp = aksjonaerer_beginning['company_share_ownership.ownership']
    aksjonaerer_mean_beginning[i]        = np.mean(temp)
    aksjonaerer_std_beginning[i]         = np.std(temp)
    aksjonaerer_max_beginning[i]         = np.max(temp)
    aksjonaerer_min_beginning[i]         = np.min(temp)
  
    aksjonaerer_andel_kommunenr[i]   = check_styre_aksj(aksjonaerer_end['kommunenr'],kommunenr)
    aksjonaerer_andel_fylkesnr[i]    = check_styre_aksj(aksjonaerer_end['fylkesnr'],fylkesnr)

print('Elapset time: {} seconds'.format(np.round(time.time()-t_start,2)))
print('Elapset time: {} minutes'.format(np.round((time.time()-t_start)/60,2)))

print('Antall årsregnskaper med to daglige ledere:{}'.format(antall_doble_daglig_leder))
print('Antall årsregnskaper med to styreledere som ser ut til å være forskjellige personer:{}'.format(antall_aarsregnskaper_med_to_forskjellige_styreledere))
print('Antall årsregnskaper uten informasjon om styre:{}'.format(antall_ingen_styre))


# Legger til variablene i data:
data['alder_dl'] = alder_dl # Alder daglig leder
data['alder_sl'] = alder_sl # Alder styrets leder
data['dummy_samme_dl_sl'] = dummy_samme_dl_sl # Dummy for om daglig leder og styreleder er samme person
data['dummy_dl_in_styre'] = dummy_dl_in_styre # Dummy for om daglig leder sitter i styret
data['dummy_dl_kjonn_K'] = dummy_dl_kjonn_K # Kjønn på daglig leder
data['dummy_sl_kjonn_K'] = dummy_sl_kjonn_K # Kjønn på styrets leder
data['dummy_to_dl'] = dummy_to_dl # Dummy for om selskapet har to daglige ledere

data['styre_alder_mean'] = styre_alder_mean # Gjenomsnittlig alder styremedlemmer
data['styre_alder_std'] = styre_alder_std # Standardavvik alder styremedlemmer
data['styre_antall'] = styre_antall # Antall styremedlemmer

data['styre_andel_kjonn_K'] = styre_andel_kjonn_K # Andel styremedlemmer som er kjønn K

data['aksjonaerer_antall_end']          = aksjonaerer_antall_end # Antall aksjonærer ved balansedato i år
data['aksjonaerer_antall_beginning']    = aksjonaerer_antall_beginning # Antall aksjonærer ved balansedato i fjor

data['aksjonaerer_mean_end']    = aksjonaerer_mean_end # Gjennomsnittlig eierandeler ved balansedato i år
data['aksjonaerer_std_end']     = aksjonaerer_std_end # Standardavvik eierandeler ved balansedato i år
data['aksjonaerer_max_end']     = aksjonaerer_max_end # Eierandel hos største aksjonær ved balansedato i år
data['aksjonaerer_min_end']     = aksjonaerer_min_end # Eierandel hos minste aksjonær ved balansedato i år

data['aksjonaerer_mean_beginning']    = aksjonaerer_mean_beginning # Gjennomsnittlig eierandeler aksjonærer ved balansedato i fjor
data['aksjonaerer_std_beginning']     = aksjonaerer_std_beginning # Standardavvik eierandeler aksjonærer ved balansedato i fjor
data['aksjonaerer_max_beginning']     = aksjonaerer_max_beginning # Eierandel hos største aksjonær ved balansedato i fjor
data['aksjonaerer_min_beginning']     = aksjonaerer_min_beginning # Eierandel hos minste aksjonær ved balansedato i fjor

data['dl_share_aksje']      = dl_share_aksje # Eierandel hos daglig leder ved årets slutt
data['sl_share_aksje']      = sl_share_aksje # Eierandel hos styreleder ved årets slutt
data['styre_share_aksje']   = styre_share_aksje # Eierandel samlet for alle som sitter i styret ved årets slutt

data['antall_styremedlemmer_ikke_aksjonaer']   = antall_styremedlemmer_ikke_aksjonaer  # Antall styremedlemmer som ikke er aksjonær
data['andel_styremedlemmer_ikke_aksjonaer']   = andel_styremedlemmer_ikke_aksjonaer # Andel styremedlemmer som ikke er aksjonær

data['dummy_dl_kommunenr']   = dummy_dl_kommunenr # Dummy: 1 dersom daglig leder har bostedsadresse med samme kommunenummer som forretningsadressen til bedriften
data['dummy_sl_kommunenr']   = dummy_sl_kommunenr # Dummy: 1 dersom styreleder har bostedsadresse med samme kommunenummer som forretningsadressen til bedriften

# Dummies fylkesnummer
data['dummy_dl_fylkesnr']           = dummy_dl_fylkesnr
data['dummy_sl_fylkesnr']           = dummy_sl_fylkesnr
data['styre_andel_fylkesnr']        = styre_andel_fylkesnr
data['aksjonaerer_andel_fylkesnr']  = aksjonaerer_andel_fylkesnr

data['styre_andel_kommunenr']           = styre_andel_kommunenr # Andel styremedlemmer som har bostedsadresse med samme kommunenummer som forretningsadressen til bedriften
data['aksjonaerer_andel_kommunenr']     = aksjonaerer_andel_kommunenr # Andel styremedlemmer som har bostedsadresse med samme kommunenummer som forretningsadressen til bedriften

# Fylkesnummer for daglig leder og styreleder
data['fnr_dl'] = fnr_dl 
data['fnr_sl'] = fnr_sl

# Saving data
data.to_csv('data_variables.csv',index=False,sep=';')