import numpy as np
# Copyright © 2005-2019, NumPy Developers.
import pandas as pd
# Copyright (c) 2008-2012, AQR Capital Management, LLC, Lambda Foundry, Inc. and PyData Development Team
import os
# from sqlalchemy import column

# laster inn data brreg
print('-----------------------------------------')
print('Loading data:')
print('-----------------------------------------')
folder_name = 'XXX/'
files = os.listdir(folder_name)
for current_file in files:
    file_year = int(current_file[0:4])

    # laster inn data
    data_loaded = pd.read_csv(folder_name+current_file,sep=';',low_memory=False)

    if current_file == files[0]:
        data_brreg = pd.DataFrame(columns=data_loaded.columns)
    data_brreg = pd.concat([data_brreg,data_loaded])
    print('Imported for accounting year {}'.format(file_year))

# Reset indeks
data_brreg = data_brreg.reset_index(drop=True)

# sjekker at alle er unik
unique_orgnr = data_brreg.groupby(['orgnr','regnaar']).size().reset_index()
temp = unique_orgnr[0].unique()
if len(temp)==1:
    print('All orgnr unique')
else:
    print('WARNING: not all orgnr unique')

# sletter
del data_loaded

# filtrerer på omsetning og sum eiendeler. for regnskapsvariabler med manglende verdier 
# betyr det at observasjonen er lik 0, bruker derfor .fillna(0)
ind = (data_brreg['sum_omsetning_EUR'].fillna(0)<=50e6) | (data_brreg['sum_eiendeler_EUR'].fillna(0)<=43e6)
ind = ind & (data_brreg['regnaar']<=2020)
ind = ind & (data_brreg['orgform']=='AS')
data_brreg = data_brreg[ind]
data_brreg = data_brreg.reset_index(drop=True) # Reset index

# laster inn data fra Enin
data_enin = pd.read_csv('data_variables.csv',sep=';',low_memory=False)

# bytter navn
data_enin.rename(columns = {'accounts.accounting_year':'regnaar'}, inplace = True)

# sjekker
ind = (data_enin['accounts_balance_sheet.total_assets']>43e6)
ind = ind & (data_enin['accounts_income_statement.total_operating_income']>50e6)
if np.sum(ind)!=0:
    print('ERROR: {} observations from Enin are not SME'.format(np.sum(ind)))
    print('However, this is probably because the accounting numbers from Enin are wrong')

data_brreg['orgnr'] = data_brreg['orgnr'].astype(int)
data_enin['orgnr'] = data_enin['orgnr'].astype(int)
data_brreg['regnaar'] = data_brreg['regnaar'].astype(int)
data_enin['regnaar'] = data_enin['regnaar'].astype(int)

# slår sammen
data_merged = data_brreg.merge(data_enin,\
    left_on=['orgnr','regnaar'],\
    right_on=['orgnr','regnaar'],\
    how='left')

if data_merged.shape[0]!=data_brreg.shape[0]:
    print('ERROR: left join failed')

if np.sum(pd.isnull(data_enin['accounts.accounting_to_date']))!=0:
    print('ERROR: accounts.accounting_to_date has missing values in data from Enin, so it cannot be used for this check')
if np.sum(pd.isnull(data_brreg['SUM EIENDELER']))!=0:
    print('ERROR: SUM EIENDELER has missing values in data from Brreg, so it cannot be used for this check')

# observasjonene i data_brreg som ikke er i data_enin, forventer 7 observasjoner
data_missing = data_merged[pd.isnull(data_merged['accounts.accounting_to_date'])]
print('Number of observations in data_brreg that are not in data_enin: {}'.format(data_missing.shape[0]))

# lagrer
def make_folder(folder_name):
    if not os.path.exists(folder_name):
        os.makedirs(folder_name)

folder_name = 'data merged'
make_folder(folder_name)

# delt i regnskapsår
for regnaar in data_merged['regnaar'].unique():
    data_merged[data_merged['regnaar']==regnaar].to_csv(folder_name+'/'+str(regnaar)+'.csv',index=False,sep=';')

# alle
data_merged.to_csv(folder_name+'/all.csv',index=False,sep=';')


