
import numpy as np 
import math


# Copyright © 2005-2019, NumPy Developers.

import pandas as pd
# Copyright (c) 2008-2012, AQR Capital Management, LLC, Lambda Foundry, Inc. and PyData Development Team

from sklearn.svm import l1_min_c
from sklearn.preprocessing import StandardScaler
import statsmodels.api as sm
# © 2007 - 2019, scikit-learn developers (BSD License).

import matplotlib.pyplot as plt
import os
import time

data = pd.read_csv('#filnavn#.csv', low_memory=False)


data['dummy_samme_dl_sl']   = data['dummy_samme_dl_sl'].astype('bool')
data['dummy_to_dl']         = data['dummy_to_dl'].astype('bool')
data['dummy_dl_kjonn_K']    = data['dummy_dl_kjonn_K'].astype('bool')
data['dummy_sl_kjonn_K']    = data['dummy_sl_kjonn_K'].astype('bool')
data['dummy_dl_fylkesnr']   = data['dummy_dl_fylkesnr'].astype('bool')
data['dummy_sl_fylkesnr']   = data['dummy_sl_fylkesnr'].astype('bool')
data['dummy_dl_in_styre']   = data['dummy_dl_in_styre'].astype('bool')
data['fravalg_revisjon']    = data['fravalg_revisjon'].astype('bool')
data['utarbeidet_regnskapsforer'] = data['utarbeidet_regnskapsforer'].astype('bool')
data['bistand_regnskapsforer'] = data['bistand_regnskapsforer'].astype('bool')


# endrer navn
data = data.rename(columns={'dummy_dl_kjonn_K':'d_dl_kjonn_k'})
data = data.rename(columns={'dummy_sl_kjonn_K':'d_sl_kjonn_k'})
data = data.rename(columns={'styre_alder_mean':'styre_alder_snitt'})
data = data.rename(columns={'dummy_sl_fylkesnr':'d_sl_fylke'})
data = data.rename(columns={'styre_andel_kjonn_K':'styre_andel_kjonn_k'})
data = data.rename(columns={'styre_andel_fylkesnr':'styre_andel_fylke'})
data = data.rename(columns={'dummy_dl_fylkesnr':'d_dl_fylke'})
data = data.rename(columns={'dummy_to_dl':'d_to_dl'})
data = data.rename(columns={'dummy_samme_dl_sl':'d_samme_dl_sl'})
data = data.rename(columns={'dummy_dl_in_styre':'d_dl_in_styre'})
data = data.rename(columns={'fravalg_revisjon':'d_fravalg_revisjon'})
data = data.rename(columns={'utarbeidet_regnskapsforer':'d_utarbeidet_regn_forer'})
data = data.rename(columns={'bistand_regnskapsforer':'d_bistand_regn_forer'})
data = data.rename(columns={'antall_styremedlemmer_ikke_aksjonaer':'antall_st_medl_ikke_aksj'})
data = data.rename(columns={'andel_styremedlemmer_ikke_aksjonaer':'andel_st_medl_ikke_aksj'})
data = data.rename(columns={'dl_share_aksje':'dl_eierandel'})
data = data.rename(columns={'sl_share_aksje':'sl_eierandel'})
data = data.rename(columns={'styre_share_aksje':'styre_eierandel'})
data = data.rename(columns={'aksjonaerer_antall_end':'aksj_antall_slutt'})
data = data.rename(columns={'aksjonaerer_std_end':'aksj_std_slutt'})
data = data.rename(columns={'aksjonaerer_max_end':'aksj_maks_slutt'})
data = data.rename(columns={'aksjonaerer_min_end':'aksj_min_slutt'})
data = data.rename(columns={'aksjonaerer_andel_fylkesnr':'aksj_andel_fylke'})


# beskrivende statistikk
besk_m_pers = data.describe(percentiles=[.01, .25, .50, .75, .99], include='all')
besk_m_pers.to_excel('#filnavn#.xlsx')

def make_variable_string1_but_if_not_string2(data,string1,string2):
    # Most of the time, string1 is correct value.
    # However, sometimes string1 has missing value while
    # string2 has the correct value
    temp = pd.DataFrame()
    temp[string1]  = data[string1]
    temp[string2]  = data[string2]
    result = pd.Series([None]*len(temp))
    for i in range(len(temp)):
        if pd.isnull(temp[string1].iloc[i])==False:
            result[i] = temp[string1].iloc[i]
        elif pd.isnull(temp[string2].iloc[i])==False:
            result[i] = temp[string2].iloc[i]
        else: # if both is 'None'
            result[i]=np.double(0)
    return pd.to_numeric(result, errors='coerce')

def make_ratio(numerator,denumerator):
    ratio = numerator.divide(denumerator)

    # If both numerator and denumerator are zero, the ratio is set to zero
    for i in range(len(ratio)):
        if numerator[i]==0:
            if denumerator[i]==0:
                ratio[i]=np.double(0)
    return ratio


def make_variables_for_predictions(data):
    bankinnskudd_kontanter_og_lignende      = make_variable_string1_but_if_not_string2(data,'Bankinnskudd, kontanter og lignende','Sum bankinnskudd, kontanter og lignende')

    skyldige_offentlige_avgifter            = data['Skyldige offentlige avgifter'].fillna(0)
    leverandorgjeld                         = data['Leverandoergjeld'].fillna(0)
    sum_kortsiktig_gjeld                    = data['Sum kortsiktig gjeld'].fillna(0)
    sum_innskutt_egenkapital                = data['Sum innskutt egenkapital'].fillna(0)
    sum_egenkapital                         = data['Sum egenkapital'].fillna(0)
    sum_eiendeler                           = data['SUM EIENDELER'].fillna(0)
    sum_gjeld                               = data['Sum gjeld'].fillna(0)
    annen_rentekostnad                      = data['Sum finanskostnader'].fillna(0)
    sum_omlopsmidler                        = data['Sum omloepsmidler'].fillna(0)
    age_in_days                             = data['age_in_days']
    alder_dl                                = data['alder_dl']
    alder_sl                                = data['alder_sl']
    styre_alder_snitt                       = data['styre_alder_snitt']
    styre_antall                            = data['styre_antall']
    aksj_antall_slutt                       = data['aksj_antall_slutt']
    nedskrivninger                          = data['Nedskrivning av varige driftsmidler og immaterielle eiendeler'].fillna(0)
    annen_renteinntekt                      = data['Annen renteinntekt'].fillna(0)
    opptjent_egenkapital                    = data['Sum opptjent egenkapital'].fillna(0)
    salgsinntekt                            = data['Salgsinntekt'].fillna(0)
    avskrivninger                           = data['Avskrivning paa varige driftsmidler og immaterielle eiendeler'].fillna(0)
    ordinaert_resultat_foer_skattekostnad   = data['Ordinaert resultat foer skattekostnad'].fillna(0)
    ordinaert_resultat_etter_skattekostnad  = data['Ordinaert resultat etter skattekostnad'].fillna(0)
    sum_inntekter                           = data['Sum inntekter'].fillna(0)
    totalkapital                            = data['SUM EIENDELER'].fillna(0)
    
    # lager variabler

    variable_list = [] 
    ### VARIABLER FRA SEBRA ###

    # RETG # SEBRA # Ordinært resultat før av- og nedskrivninger i prosent av total gjeld
    name = 'res_gjeld'
    variable_list.append(name)
    temp = ordinaert_resultat_etter_skattekostnad + avskrivninger + nedskrivninger
    data[name] = make_ratio(temp,sum_gjeld)

    # EKTK # SEBRA # Egenkapital i prosent av totalkapital
    name = 'ek_tk'
    variable_list.append(name)
    data[name] = make_ratio(sum_egenkapital,totalkapital)

    # DEK # SEBRA # Dummy-variabel: bokført egenkapital er mindre enn innskutt egenkapital 
    name = 'd_bek_iek'
    variable_list.append(name)
    data[name] = sum_egenkapital < sum_innskutt_egenkapital

    # LIKOMS # SEBRA # Likvide midler minus kortsikig gjeld i prosent av omsetning
    name = 'likv_kg_oms'
    variable_list.append(name)
    temp = bankinnskudd_kontanter_og_lignende - sum_kortsiktig_gjeld
    data[name] = make_ratio(temp,sum_inntekter)

    # logALDER # SEBRA # Logaritmen av år siden etablering
    name = 'log_alder'
    variable_list.append(name)
    # Handling negative age
    ind = age_in_days<=0
    print('{} observations have negative age. Age set to zero for these.'.format(np.sum(ind)))
    age_in_days.loc[ind] = 1
    # Handling missing age
    ind = pd.isnull(age_in_days)
    print('{} observations have missing age. Age set to zero for these.'.format(np.sum(ind)))
    age_in_days.loc[ind] = 1
    data[name] = np.log(data['age_in_days']/365)

    # logTK # SEBRA # Logaritmen av totalkapital i faste kroner
    name = 'log_tk'
    variable_list.append(name)
    data[name] = np.log(totalkapital)

    # LGTK # SEBRA # Leverandørgjeld i prosent av totalkapital
    name = 'levgjeld_tk'
    variable_list.append(name)
    data[name] = make_ratio(leverandorgjeld,totalkapital)
    
    # OATK # SEBRA # Skyldige offentlige avgifter i prosent av totalkapital
    name = 'avg_tk'
    variable_list.append(name)
    data[name] = make_ratio(skyldige_offentlige_avgifter,totalkapital)
 
    ### ALTMAN 1968, med X4 fra Z' 2000 ###

    # X1 # ALTMAN 1968 # Arbeidskapital / Sum eiendeler
    name = 'arbkap_tk'
    variable_list.append(name)
    temp = sum_omlopsmidler - sum_kortsiktig_gjeld
    data[name] = make_ratio(temp,sum_eiendeler)

    # X2 # ALTMAN 1968 # Opptjent egenkapital / Sum eiendeler
    name = 'opptj_ek_tk'
    variable_list.append(name)
    data[name] = make_ratio(opptjent_egenkapital,sum_eiendeler)

    # X3 # ALTMAN 1968 # EBIT / Sum eiendeler
    name = 'EBIT_tk'
    variable_list.append(name)
    temp = ordinaert_resultat_foer_skattekostnad + annen_rentekostnad - annen_renteinntekt
    data[name] = make_ratio(temp,sum_eiendeler)

    # X4 # ALTMAN 2000 # Sum egenkapital / Sum gjeld
    name = 'ek_gjeld'
    variable_list.append(name)
    data[name] = make_ratio(sum_egenkapital,sum_gjeld)

    # X5 # ALTMAN 1968 # Salgsinntekter / Sum eiendeler
    name = 'salginntekt_tk'
    variable_list.append(name)
    data[name] = make_ratio(salgsinntekt,sum_eiendeler)

    ##### IKKE - FINANSIELLE VARIABLER
    name = 'log_alder_dl'
    variable_list.append(name)
    data[name] = np.log(alder_dl)

    name = 'log_alder_sl'
    variable_list.append(name)
    data[name] = np.log(alder_sl)

    name = 'log_styre_alder_snitt'
    variable_list.append(name)
    data[name] = np.log(styre_alder_snitt)

    name = 'log_styre_antall'
    variable_list.append(name)
    data[name] = np.log(styre_antall)

    name = 'log_aksj_antall_slutt'
    variable_list.append(name)
    data[name] = np.log(aksj_antall_slutt)

    name = 'd_fravalg_revisjon'
    variable_list.append(name)

    name = 'd_utarbeidet_regn_forer'
    variable_list.append(name)

    name = 'd_bistand_regn_forer'
    variable_list.append(name)
    
    name = 'd_samme_dl_sl'
    variable_list.append(name)
    
    name = 'd_dl_in_styre'
    variable_list.append(name)
    
    name = 'd_dl_kjonn_k'
    variable_list.append(name)
    
    name = 'd_sl_kjonn_k'
    variable_list.append(name)
    
    name = 'd_to_dl'
    variable_list.append(name)
    
    name = 'styre_alder_std'
    variable_list.append(name)
    
    name = 'styre_andel_kjonn_k'
    variable_list.append(name)
    
    name = 'dl_eierandel'
    variable_list.append(name)
    
    name = 'sl_eierandel'
    variable_list.append(name)
    
    name = 'styre_eierandel'
    variable_list.append(name)
    
    name = 'andel_st_medl_ikke_aksj'
    variable_list.append(name)
    
    name = 'd_dl_fylke'
    variable_list.append(name)
    
    name = 'd_sl_fylke'
    variable_list.append(name)

    name = 'styre_andel_fylke'
    variable_list.append(name)

    name = 'aksj_std_slutt'
    variable_list.append(name)

    name = 'aksj_maks_slutt'
    variable_list.append(name)

    name = 'aksj_min_slutt'
    variable_list.append(name)
    
    return data,variable_list


def handle_inf(column):
    column = column.replace(np.inf,np.max(column[column != np.inf]))
    column = column.replace(-np.inf,np.max(column[column != -np.inf]))
    return column

def winsorize_rrw(column,interval_winsorizing):
    lower=column.quantile(interval_winsorizing[0])
    upper=column.quantile(interval_winsorizing[1])
    return column.apply(lambda x: winsorize_rrw_inner(x,lower,upper))
    # return data.clip(lower=data.quantile(interval_winsorizing[0]), upper=data.quantile(interval_winsorizing[1]))

def winsorize_rrw_inner(x,lower,upper):
    if x<=lower:
        return lower
    elif x>=upper:
        return upper
    else:
        return x

def winsorizing_and_handling_inf(data,interval_winsorizing,variable_list):
    ############################################
    # First, handling Inf and -Inf by setting them to max and min, respectively
    ############################################
    for name in variable_list:
        data[name] = handle_inf(data[name])

    # Checking that handling Inf and -Inf went well
    for name in variable_list:
        column = data[name]
        if ((column == np.inf).sum()!=0):
            print('ERROR: some observations of variable {} are still inf.'.format(name))
        if ((column == -np.inf).sum()!=0):
            print('ERROR: some observations of variable {} are still -inf.'.format(name))

    ############################################
    # Second, winsorizing
    ############################################

    for name in variable_list:
        if data[name].dtype!='bool': # Skipping dummies
            data[name] = winsorize_rrw(data[name],interval_winsorizing)


    # Checking that winsorizing went well
    for name in variable_list:
        temp = data[name]
        if temp.dtype!='bool': # Skipping dummies
             # First, checking lower quantile of winsorizing:
            temp1   = temp.quantile([interval_winsorizing[0]]).iloc[0]
            temp2   = temp.quantile([interval_winsorizing[0]/10]).iloc[0]
            if temp1!=temp2:
                pst_change = np.abs((temp1-temp2)/temp1)
                if pst_change>1e3: # Allow for up to 0.1% difference due to acceptable computational errors
                    print('ERROR: winsorizing failed for variable {}.'.format(name))
            # Second, checking upper quantile of winsorizing:
            temp1       = temp.quantile([interval_winsorizing[1]]).iloc[0]
            temp_upper  = interval_winsorizing[1]*(1+1-interval_winsorizing[1])
            temp2       = temp.quantile([temp_upper]).iloc[0]
            if temp1!=temp2:
                pst_change = np.abs((temp1-temp2)/temp1)
                if pst_change>1e3: # Allow for up to 0.1% difference due to acceptable computational errors
                    print('ERROR: winsorizing failed for variable {}.'.format(name))

    return data

def making_folds(first_regnaar,num_years_rolling_window,data):
    # Making df with all observations per regnaar
    unique_regnaar = np.sort(data.regnaar.unique())
    all_observations_per_regnaar = pd.DataFrame(columns = ['data'],index=unique_regnaar)
    for regnaar in unique_regnaar:
        all_observations_per_regnaar.loc[regnaar] = {'data' : data[data['regnaar'] == regnaar]}

    # Making folds
    regnaar_for_test = unique_regnaar[unique_regnaar>=first_regnaar]
    data_folds = pd.DataFrame(columns = ['training_data','testing_data'],index=regnaar_for_test)
    for regnaar_test in regnaar_for_test:
        if (regnaar_test in unique_regnaar)==False:
            print('ERROR: Trying to use regnaar {} as testing data, but it does not exist'.format(regnaar_test))
        else:
            testing_data = all_observations_per_regnaar.loc[regnaar_test]['data']
            regnaar_for_train = range(regnaar_test-num_years_rolling_window,regnaar_test)
            for regnaar_train in regnaar_for_train:
                if (regnaar_train in unique_regnaar)==False:
                    print('ERROR: Trying to use regnaar {} in training data, but it does not exist'.format(regnaar_train))
                else:
                    if regnaar_train == np.min(regnaar_for_train):
                        training_data=all_observations_per_regnaar.loc[regnaar_train]['data']
                    else:
                        training_data = pd.concat([training_data,all_observations_per_regnaar.loc[regnaar_train]['data']])
        testing_data = testing_data.reset_index(drop=True) # Reset index
        training_data = training_data.reset_index(drop=True) # Reset index
        data_folds.loc[regnaar_test] = {'training_data':training_data,'testing_data':testing_data}

    # Checking

    for i in data_folds.index:
        testing_data            = data_folds.loc[i]['testing_data']
        training_data           = data_folds.loc[i]['training_data']
        unique_regnaar_test     = testing_data['regnaar'].unique()
        unique_regnaar_train    = training_data['regnaar'].unique()
        if len(unique_regnaar_test)!=1:
            print('ERROR')
        if len(unique_regnaar_train)!=num_years_rolling_window:
            print('ERROR')
        if np.sum(unique_regnaar_train>=unique_regnaar_test):
            print('ERROR: Data leakage')

    return data_folds

def get_training_and_testing_data_for_fold(training_data,testing_data,variable_list):
    # Dividing between X (independent variables) and y (dependent variable)
    if np.sum(np.sum(pd.isnull(training_data[variable_list])))!=0:
        print('ERROR: missing values in training data')
    if np.sum(np.sum(pd.isnull(training_data['bankrupt_fs'])))!=0:
        print('ERROR: missing values in training data')

    if np.sum(np.sum(pd.isnull(testing_data[variable_list])))!=0:
        print('ERROR: missing values in training data')
    if np.sum(np.sum(pd.isnull(testing_data['bankrupt_fs'])))!=0:
        print('ERROR: missing values in training data')

    # Making training data (into ndarray)
    X_train = training_data[variable_list]
    y_train = training_data['bankrupt_fs']
    X_train = X_train.astype(float)
    y_train = y_train.astype(int)

    # Making test data (into ndarray)
    X_test = testing_data[variable_list]
    y_test = testing_data['bankrupt_fs']
    X_test = X_test.astype(float)   
    y_test = y_test.astype(int)     

    return X_train, y_train, X_test, y_test


data,variable_list = make_variables_for_predictions(data)

# ulike variabler tas ut av variabellisten, alt etter type variabelsett

# sletter aksjonærvariabler og revisjon/regnskap, inkluderes i egen analyse 2014 - 2020
#variable_list.remove('dl_eierandel')
#variable_list.remove('sl_eierandel')
#variable_list.remove('styre_eierandel')
#variable_list.remove('aksj_snitt_slutt')
#variable_list.remove('aksj_std_slutt')
#variable_list.remove('aksj_maks_slutt')
#variable_list.remove('aksj_min_slutt')
#variable_list.remove('andel_st_medl_ikke_aksj')
#variable_list.remove('styre_andel_fylke')
#variable_list.remove('d_dl_fylke')
#variable_list.remove('d_sl_fylke')
#variable_list.remove('d_fravalg_revisjon')
#variable_list.remove('d_bistand_regn_forer')
#variable_list.remove('d_utarbeidet_regn_forer')

# øvrige ikke-finansielle variabler
#variable_list.remove('d_samme_dl_sl')
#variable_list.remove('d_dl_in_styre')
#variable_list.remove('d_dl_kjonn_k')
#variable_list.remove('d_sl_kjonn_k')
#variable_list.remove('d_to_dl')
#variable_list.remove('styre_alder_std')
#variable_list.remove('styre_andel_kjonn_k')


#SEBRA
#variable_list.remove('res_gjeld')
#variable_list.remove('ek_tk')
#variable_list.remove('d_bek_iek')
#ariable_list.remove('likv_kg_oms')
#variable_list.remove('log_alder')
#variable_list.remove('log_tk')
#ariable_list.remove('levgjeld_tk')
#variable_list.remove('avg_tk')


# Altman
#variable_list.remove('arbkap_tk')
#variable_list.remove('opptj_ek_tk')
#variable_list.remove('EBIT_tk')
#variable_list.remove('ek_gjeld')
#variable_list.remove('salginntekt_tk')



# tar log-variabler ut av variabellisten, slik at de ikke blir både logges og winsorizes

variable_list.remove('log_alder_dl')
variable_list.remove('log_alder_sl')
variable_list.remove('log_styre_alder_snitt')
variable_list.remove('log_styre_antall')
variable_list.remove('log_aksj_antall_slutt')
variable_list.remove('log_tk')
variable_list.remove('log_alder')

interval_winsorizing = [0.01,0.99] # In numbers, so 0.01 = restricting at 1%
data = winsorizing_and_handling_inf(data,interval_winsorizing,variable_list)


# sette tilbake log-variabler i variabellisten etter at winsorising er gjennomført

variable_list.append('log_alder_dl')
variable_list.append('log_alder_sl')
variable_list.append('log_styre_alder_snitt')
variable_list.append('log_styre_antall')
variable_list.append('log_aksj_antall_slutt')
variable_list.append('log_alder')
variable_list.append('log_tk')


first_regnaar = 2010 # First accounting year to use for test
num_years_rolling_window = 4 # Number of accounting years to use for training data
data_folds = making_folds(first_regnaar,num_years_rolling_window,data)


for regnaar_test in data_folds.index:

    # Making training and test data
    training_data   = data_folds.loc[regnaar_test]['training_data']
    testing_data    = data_folds.loc[regnaar_test]['testing_data']

    # Shuffeling rows
    training_data   = training_data.sample(frac=1,random_state=0).reset_index(drop=True)
    testing_data    = testing_data.sample(frac=1,random_state=0).reset_index(drop=True)

    # Dividing between X (independent variables) and y (dependent variable)
    X_train, y_train, X_test, y_test = get_training_and_testing_data_for_fold(training_data,testing_data,variable_list)
    
    X_train.to_csv('x_train_modell_XX'+str(regnaar_test)+'.csv')
    y_train.to_csv('y_train_modell_XX'+str(regnaar_test)+'.csv')
    X_test.to_csv('x_test_modell_XX'+str(regnaar_test)+'.csv')
    y_test.to_csv('y_test_modell_XX'+str(regnaar_test)+'.csv')
